<?php
// $Id: page_theme.admin.inc,v 1.1 2009/04/29 22:40:29 flanker Exp $
// $Name: DRUPAL-6--1-0-RC1 $

/**
 * @file
 *   Administrative page callbacks for the page theme module.
 */


/**
 * Menu callback; lists all defined page themes.
 */
function page_theme_admin_list() {

  $themes = page_theme_get_themes();

  $header = array(t('Theme'), t('Pages'), array('data' => t('Operations'), 'colspan' => '2'));
  $rows = array();
  $result = db_query('SELECT * FROM {page_theme} ORDER BY theme');
  while ($theme = db_fetch_object($result)) {
    $row = array();
    $row[] = $themes[$theme->theme];
    $row[] = str_replace("\n", '<br />', check_plain($theme->pages));
    $row[] = l(t('edit'), 'admin/build/page-theme/edit/'. $theme->theme);
    $row[] = l(t('delete'), 'admin/build/page-theme/delete/'. $theme->theme);
    $rows[] = $row;
  }
  if (empty($rows)) {
    $rows[] = array(array('data' => t('No page themes defined.'), 'colspan' => 4));
  }

  $output = theme('table', $header, $rows);

  return $output;
}


/**
 * Menu callback; adds a page theme.
 */
function page_theme_admin_add() {

  $form = array();
  $form['theme'] = array(
    '#type' => 'select',
    '#title' => t('Theme'),
    '#description' => t('Choose which theme the page should display in. If you choose "Site theme" the pages will use the same theme as the rest of the site.'),
    '#default_value' => '0',
    '#options' => page_theme_get_theme_options(),
    '#required' => TRUE,
  );
  $form['pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Pages'),
    '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>')),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create page theme'),
  );

  return $form;
}

function page_theme_admin_add_validate($form, &$form_state) {

  if (empty($form_state['values']['theme'])) {
    form_set_error('theme', t('Please select a theme.'));
  }
}

function page_theme_admin_add_submit($form, &$form_state) {

  $theme = $form_state['values']['theme'];
  $pages = trim($form_state['values']['pages']);

  db_query('DELETE FROM {page_theme} WHERE theme = "%s"', $theme);
  db_query('INSERT INTO {page_theme} (pages, theme) VALUES ("%s", "%s")', $pages, $theme);

  drupal_set_message(t('The page theme has been saved.'));
  $form_state['redirect'] = 'admin/build/page-theme';
}


/**
 * Menu callback; edits a page theme.
 */
function page_theme_admin_edit($form_state, $theme = NULL) {

  if (is_null($theme)) {
    drupal_goto('admin/build/page-theme');
  }

  $themes = page_theme_get_themes();
  $pages = db_result(db_query('SELECT pages FROM {page_theme} WHERE theme = "%s"', $theme));

  $form = array();
  $form['theme'] = array(
    '#type' => 'value',
    '#value' => $theme,
  );
  $form['theme_name'] = array(
    '#type' => 'item',
    '#title' => t('Theme'),
    '#value' => $themes[$theme],
  );
  $form['pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Pages'),
    '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>')),
    '#default_value' => $pages,
    '#required' => TRUE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update page theme'),
  );

  return $form;
}

function page_theme_admin_edit_submit($form, &$form_state) {

  $theme = $form_state['values']['theme'];
  $pages = trim($form_state['values']['pages']);

  db_query('DELETE FROM {page_theme} WHERE theme = "%s"', $theme);
  db_query('INSERT INTO {page_theme} (pages, theme) VALUES ("%s", "%s")', $pages, $theme);

  drupal_set_message(t('The page theme has been updated.'));
  $form_state['redirect'] = 'admin/build/page-theme';
}


/**
 * Menu callback; deletes a page theme.
 */
function page_theme_admin_delete($form_state, $theme = NULL) {

  if (is_null($theme)) {
    drupal_goto('admin/build/page-theme');
  }

  $form = array();
  $form['theme'] = array(
    '#type' => 'value',
    '#value' => $theme,
  );
  $themes = page_theme_get_themes();
  $question = t('Are you sure you want to delete page theme of %theme?', array('%theme' => $themes[$theme]));
  $path = isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/page-theme';

  return confirm_form($form, $question, $path);
}

function page_theme_admin_delete_submit($form, &$form_state) {

  if ($form_state['values']['confirm']) {
    db_query('DELETE FROM {page_theme} WHERE theme = "%s"', $form_state['values']['theme']);

    drupal_set_message(t('The page theme has been deleted.'));
    $form_state['redirect'] = 'admin/build/page-theme';
    return;
  }
}


/**
 *  Helper functions.
 */
function page_theme_get_theme_options() {
  static $options = array();

  if (!$options) {
    $themes = page_theme_get_themes();

    $result = db_query('SELECT theme FROM {page_theme}');
    while ($theme = db_fetch_object($result)) {
      unset($themes[$theme->theme]);
    }

    $options[0] = '- '. t('Select theme') .' -';
    $options += $themes;
  }

  return $options;
}

function page_theme_get_themes() {

  $theme_data = system_theme_data();
  uasort($theme_data, 'page_theme_sort_themes');

  $themes = array();
  foreach ($theme_data as $theme) {
    $themes[$theme->name] = $theme->info['name'];
  }

  return $themes;
}

function page_theme_sort_themes($a, $b) {
  return strcasecmp($a->info['name'], $b->info['name']);
}
